==> ml_algo-15.6.5.tar.gz: lib/src/tree_trainer/decision_tree_trainer.dart <==
import 'package:ml_algo/src/tree_trainer/leaf_detector/leaf_detector.dart';
import 'package:ml_algo/src/tree_trainer/leaf_label/leaf_label_factory.dart';
import 'package:ml_algo/src/tree_trainer/split_selector/split_selector.dart';
import 'package:ml_algo/src/tree_trainer/tree_node/splitting_predicate/tree_node_splitting_predicate_type.dart';
import 'package:ml_algo/src/tree_trainer/tree_node/tree_node.dart';
import 'package:ml_algo/src/tree_trainer/tree_trainer.dart';
import 'package:ml_linalg/matrix.dart';

class DecisionTreeTrainer implements TreeTrainer {
  DecisionTreeTrainer(
      this._featureIndices,
      this._targetIdx,
      this._featureToUniqueValues,
      this._leafDetector,
      this._leafLabelFactory,
      this._splitSelector,
  );
  
  final Iterable<int> _featureIndices;
  final int _targetIdx;
  final Map<int, List<num>> _featureToUniqueValues;
  final TreeLeafDetector _leafDetector;
  final TreeLeafLabelFactory _leafLabelFactory;
  final TreeSplitSelector _splitSelector;

  @override
  TreeNode train(Matrix samples) =>
      _train(samples, null, null, null, _featureIndices, 0);

  TreeNode _train(
      Matrix samples,
      num splittingValue,
      int splittingIdx,
      TreeNodeSplittingPredicateType splittingPredicateType,
      Iterable<int> featuresColumnIdxs,
      int level,
  ) {
    if (_leafDetector.isLeaf(samples, _targetIdx, featuresColumnIdxs, level)) {
      final label = _leafLabelFactory.create(samples, _targetIdx);
      return TreeNode(
        splittingPredicateType,
        splittingValue,
        splittingIdx,
        null,
        label,
        level,
      );
    }

    final bestSplit = _splitSelector.select(
        samples,
        _targetIdx,
        featuresColumnIdxs,
        _featureToUniqueValues,
    );

    final newLevel = level + 1;

    final childNodes = bestSplit.entries.map((entry) {
      final splitNode = entry.key;
      final splitSamples = entry.value;

      final isSplitByNominalValue = _featureToUniqueValues
          .containsKey(splitNode.splittingIndex);

      final updatedColumnRanges = isSplitByNominalValue
          ? (Set<int>.from(featuresColumnIdxs)
              ..remove(splitNode.splittingIndex))
          : featuresColumnIdxs;

      return _train(
          splitSamples,
          splitNode.splittingValue,
          splitNode.splittingIndex,
          splitNode.predicateType,
          updatedColumnRanges,
          newLevel);
    });

    return TreeNode(
        splittingPredicateType,
        splittingValue,
        splittingIdx,
        childNodes.toList(growable: false),
        null,
        level);
  }
}

==> ml_algo-16.0.0.tar.gz: lib/src/tree_trainer/decision_tree_trainer.dart <==
import 'package:ml_algo/src/tree_trainer/leaf_detector/leaf_detector.dart';
import 'package:ml_algo/src/tree_trainer/leaf_label/leaf_label_factory.dart';
import 'package:ml_algo/src/tree_trainer/split_selector/split_selector.dart';
import 'package:ml_algo/src/tree_trainer/tree_node/splitting_predicate/tree_node_splitting_predicate_type.dart';
import 'package:ml_algo/src/tree_trainer/tree_node/tree_node.dart';
import 'package:ml_algo/src/tree_trainer/tree_trainer.dart';
import 'package:ml_linalg/matrix.dart';

class DecisionTreeTrainer implements TreeTrainer {
  DecisionTreeTrainer(
      this._featureIndices,
      this._targetIdx,
      this._featureToUniqueValues,
      this._leafDetector,
      this._leafLabelFactory,
      this._splitSelector,
  );
  
  final Iterable<int> _featureIndices;
  final int _targetIdx;
  final Map<int, List<num>> _featureToUniqueValues;
  final TreeLeafDetector _leafDetector;
  final TreeLeafLabelFactory _leafLabelFactory;
  final TreeSplitSelector _splitSelector;

  @override
  TreeNode train(Matrix samples) =>
      _train(samples, null, null, null, _featureIndices, 0);

  TreeNode _train(
      Matrix samples,
      num? splittingValue,
      int? splittingIdx,
      TreeNodeSplittingPredicateType? splittingPredicateType,
      Iterable<int> featuresColumnIdxs,
      int level,
  ) {
    final isLeaf = _leafDetector.isLeaf(
      samples,
      _targetIdx,
      featuresColumnIdxs,
      level,
    );

    if (isLeaf) {
      final label = _leafLabelFactory.create(
        samples,
        _targetIdx,
      );

      return TreeNode(
        splittingPredicateType,
        splittingValue,
        splittingIdx,
        null,
        label,
        level,
      );
    }

    final bestSplit = _splitSelector.select(
        samples,
        _targetIdx,
        featuresColumnIdxs,
        _featureToUniqueValues,
    );

    final newLevel = level + 1;

    final childNodes = bestSplit.entries.map((entry) {
      final splitNode = entry.key;
      final splitSamples = entry.value;
      final isSplitByNominalValue = _featureToUniqueValues
          .containsKey(splitNode.splittingIndex);
      final updatedColumnRanges = isSplitByNominalValue
          ? (Set<int>.from(featuresColumnIdxs)
              ..remove(splitNode.splittingIndex))
          : featuresColumnIdxs;

      return _train(
        splitSamples,
        splitNode.splittingValue,
        splitNode.splittingIndex,
        splitNode.predicateType,
        updatedColumnRanges,
        newLevel,
      );
    });

    return TreeNode(
      splittingPredicateType,
      splittingValue,
      splittingIdx,
      childNodes.toList(growable: false),
      null,
      level,
    );
  }
}
